#include <Wire.h>
#include <EEPROM.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

//replace carbon filter every ~640 hours (found some google search references about activated carbon filter replacement suggesting this runtime)

//Output Pins
#define LED 13    //debug LED indicator
#define SDA A4    //I2C data for oled
#define SCL A5    //I2C clock for oled
#define RST 6     //turn on/off oled display
#define Vref   5  //turn on/off diode reference
#define Vdiode A0 //measure 0.7V diode reference to calculate battery supply voltage
#define PWM    9  //modulate fan speed
#define Vboost 4  //turn on/off 12V boost converter
//#define SPKp   7  //Buzzer positive
//#define SPKn   8  //Buzzer negative

//Input Pins
#define PWR 3//2     //power/menu button
#define B1  A2    //up button
#define B2  A1    //down button
#define CHG 2//3     //Charging indication input

#define OLEDaddr 0x3C //oled I2C address
#define CMD 0         //command packet
#define DAT 0x40      //data (image) packet

//Display in normal or inverted pixels
#define NORM 0
#define INV  1

//Icon Indices
#define Fan0   0
#define Fan1   1
#define Filter 2
#define Bat4   0
#define Bat3   1
#define Bat2   2
#define Bat1   3
#define Low    4
#define Charge 6

//Global Variables
unsigned char Speed=6;      //fan speed from 1-10 (mapped to 1-255 pwm)
unsigned long totaltime=0;  //total time elapsed while on
bool sleep_flg=1;           //1, sw request to sleep, 1st boot into sleep mode

uint8_t fan_speed[]={12,19,25,31,38,44,50,76,101,127}; //12,25,38,50,63,76,88,101,114,127

void wakeInterrupt(void){
  detachInterrupt(1); //disable power button interrupt
  digitalWrite(LED,HIGH);
  digitalWrite(LED,LOW);
}

void chargeInterrupt(void){
  detachInterrupt(0); //disable charge indicator input interrupt
}

const char SmallFont[] PROGMEM = {0x05,0x08,  //5x7 Small 8 bit Font
                                0x00,0x00,0x00,0x00,0x00, // ' '  , 480 bytes
                                0x00,0x00,0x5F,0x00,0x00, // !
                                0x00,0x07,0x00,0x07,0x00, // "
                                0x14,0x7F,0x14,0x7F,0x14, // #
                                0x24,0x2A,0x7F,0x2A,0x12, // $
                                0x23,0x13,0x08,0x64,0x62, // %
                                0x36,0x49,0x55,0x22,0x50, // &
                                0x00,0x05,0x03,0x00,0x00, // '
                                0x00,0x1C,0x22,0x41,0x00, // (
                                0x00,0x41,0x22,0x1C,0x00, // )
                                0x14,0x08,0x3E,0x08,0x14, // *
                                0x08,0x08,0x3E,0x08,0x08, // +
                                0x00,0x50,0x30,0x00,0x00, // ,
                                0x08,0x08,0x08,0x08,0x08, // -
                                0x00,0x60,0x60,0x00,0x00, // .
                                0x40,0x20,0x10,0x08,0x04, // /
              
                                0x3E,0x51,0x49,0x45,0x3E, //0
                                0x00,0x42,0x7F,0x40,0x00, //1
                                0x42,0x61,0x51,0x49,0x46, //2
                                0x22,0x49,0x49,0x49,0x36, //3
                                0x18,0x14,0x12,0x7F,0x10, //4
                                0x27,0x45,0x45,0x45,0x39, //5
                                0x3E,0x49,0x49,0x49,0x32, //6
                                0x01,0x01,0x71,0x09,0x07, //7
                                0x36,0x49,0x49,0x49,0x36, //8
                                0x26,0x49,0x49,0x49,0x3E, //9
                                0x00,0x36,0x36,0x00,0x00, // :
                                0x00,0x56,0x36,0x00,0x00, // ;
                                0x08,0x14,0x22,0x41,0x00, // <
                                0x14,0x14,0x14,0x14,0x14, // =
                                0x41,0x22,0x14,0x08,0x00, // >
                                0x02,0x01,0x51,0x09,0x06, // ?
             
                                0x32,0x49,0x79,0x41,0x3E, // @
                                0x7C,0x12,0x11,0x12,0x7C, //A
                                0x7F,0x49,0x49,0x49,0x36, //B
                                0x3E,0x41,0x41,0x41,0x22, //C
                                0x7F,0x41,0x41,0x41,0x3E, //D
                                0x7F,0x49,0x49,0x49,0x41, //E
                                0x7F,0x09,0x09,0x09,0x01, //F
                                0x3E,0x41,0x49,0x49,0x7A, //G
                                0x7F,0x08,0x08,0x08,0x7F, //H
                                0x00,0x41,0x7F,0x41,0x00, //I
                                0x20,0x41,0x41,0x3F,0x01, //J
                                0x7F,0x08,0x14,0x22,0x41, //K
                                0x7F,0x40,0x40,0x40,0x40, //L
                                0x7F,0x02,0x0C,0x02,0x7F, //M
                                0x7F,0x04,0x08,0x10,0x7F, //N
                                0x3E,0x41,0x41,0x41,0x3E, //O
            
                                0x7F,0x09,0x09,0x09,0x06, //P
                                0x3E,0x41,0x51,0x21,0x5E, //Q
                                0x7F,0x09,0x19,0x29,0x46, //R
                                0x26,0x49,0x49,0x49,0x32, //S
                                0x01,0x01,0x7F,0x01,0x01, //T
                                0x3F,0x40,0x40,0x40,0x3F, //U
                                0x1F,0x20,0x40,0x20,0x1F, //V
                                0x3F,0x40,0x38,0x40,0x3F, //W
                                0x63,0x14,0x08,0x14,0x63, //X
                                0x07,0x08,0x70,0x08,0x07, //Y
                                0x61,0x51,0x49,0x45,0x43, //Z
                                0x00,0x7F,0x41,0x41,0x00, // [
                                0x15,0x16,0x7C,0x16,0x15, // Yen
                                0x00,0x41,0x41,0x7F,0x00, // ]
                                0x04,0x02,0x01,0x02,0x04, // ^
                                0x40,0x40,0x40,0x40,0x40, // _
            
                                0x00,0x01,0x02,0x04,0x00, // `
                                0x20,0x54,0x54,0x54,0x78, //a
                                0x7F,0x48,0x44,0x44,0x38, //b
                                0x38,0x44,0x44,0x44,0x20, //c
                                0x38,0x44,0x44,0x48,0x7F, //d
                                0x38,0x54,0x54,0x54,0x18, //e
                                0x08,0x7E,0x09,0x01,0x02, //f
                                0x0C,0x52,0x52,0x52,0x3E, //g
                                0x7F,0x08,0x04,0x04,0x78, //h
                                0x00,0x44,0x7D,0x40,0x00, //i
                                0x20,0x40,0x44,0x3D,0x00, //j
                                0x7F,0x10,0x28,0x44,0x00, //k
                                0x00,0x41,0x7F,0x40,0x00, //l
                                0x7C,0x04,0x18,0x04,0x78, //m
                                0x7C,0x08,0x04,0x04,0x78, //n
                                0x38,0x44,0x44,0x44,0x38, //o
           
                                0x7C,0x14,0x14,0x14,0x08, //p
                                0x08,0x14,0x14,0x18,0x7C, //q
                                0x7C,0x08,0x04,0x04,0x08, //r
                                0x48,0x54,0x54,0x54,0x24, //s
                                0x04,0x3F,0x44,0x40,0x20, //t
                                0x3C,0x40,0x40,0x20,0x7C, //u
                                0x1C,0x20,0x40,0x20,0x1C, //v
                                0x3C,0x40,0x38,0x40,0x3C, //w
                                0x44,0x28,0x10,0x28,0x44, //x
                                0x0C,0x50,0x50,0x50,0x3C, //y
                                0x44,0x64,0x54,0x4C,0x44, //z
                                0x00,0x08,0x36,0x41,0x00, // {
                                0x00,0x00,0x7F,0x00,0x00, // |
                                0x00,0x41,0x36,0x08,0x00, // }
                                0x08,0x08,0x2A,0x1C,0x08, // ->
                                0x08,0x1C,0x2A,0x08,0x08};// <-

const char BigFont[] PROGMEM = {0x0A,0x10, //10x14 Large Smooth Font, width x height in pixels
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,  //' '
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                              0x00,0x00,0x00,0x00,0xFE,0xFE,0x00,0x00,0x00,0x00,  // !
                              0x00,0x00,0x00,0x00,0x67,0x67,0x00,0x00,0x00,0x00,
                              0x00,0x00,0x3F,0x3F,0x00,0x00,0x3F,0x3F,0x00,0x00,  // "
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                              0x60,0x60,0xFE,0xFE,0x60,0x60,0xFE,0xFE,0x60,0x60,  // #
                              0x06,0x06,0x7F,0x7F,0x06,0x06,0x7F,0x7F,0x06,0x06,
                              0x60,0xF0,0xF8,0x98,0xFE,0xFE,0x98,0x98,0x98,0x18,  // $
                              0x18,0x18,0x19,0x19,0x7F,0x7F,0x19,0x1F,0x0F,0x06,
                              0x1C,0x36,0x22,0x36,0x9C,0xC0,0xE0,0x70,0x38,0x18,  // %
                              0x18,0x1C,0x0E,0x07,0x03,0x39,0x6C,0x44,0x6C,0x38,
                              0x78,0xFC,0x8E,0xC6,0xE6,0x6E,0x3C,0x18,0x00,0x00,  // &
                              0x1E,0x3F,0x71,0x63,0x67,0x6E,0x3C,0x18,0x7E,0x66,
                              0x00,0x00,0x66,0x76,0x3E,0x1E,0x00,0x00,0x00,0x00,  // '
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                              0x00,0x00,0xE0,0xF0,0x38,0x1C,0x0E,0x06,0x00,0x00,  // (
                              0x00,0x00,0x07,0x0F,0x1C,0x38,0x70,0x60,0x00,0x00,
                              0x00,0x00,0x06,0x0E,0x1C,0x38,0xF0,0xE0,0x00,0x00,  // )
                              0x00,0x00,0x60,0x70,0x38,0x1C,0x0F,0x07,0x00,0x00,
                              0x30,0x70,0xE0,0xC0,0xF8,0xF8,0xC0,0xE0,0x70,0x30,  // *
                              0x18,0x1C,0x0E,0x07,0x3F,0x3F,0x07,0x0E,0x1C,0x18,
                              0x80,0x80,0x80,0x80,0xF8,0xF8,0x80,0x80,0x80,0x80,  // +
                              0x01,0x01,0x01,0x01,0x1F,0x1F,0x01,0x01,0x01,0x01,
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,  // ,
                              0x00,0x00,0x66,0x76,0x3E,0x1E,0x00,0x00,0x00,0x00,
                              0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,  // -
                              0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,  // .
                              0x00,0x00,0x00,0x30,0x78,0x78,0x30,0x00,0x00,0x00,
                              0x00,0x00,0x00,0x00,0x80,0xC0,0xE0,0x70,0x38,0x18,  // /
                              0x18,0x1C,0x0E,0x07,0x03,0x01,0x00,0x00,0x00,0x00,

                              0xF8,0xFC,0x0E,0x06,0x86,0xC6,0xE6,0x7E,0xFC,0xF8,  // 0
                              0x1F,0x3F,0x7E,0x67,0x63,0x61,0x60,0x70,0x3F,0x1F,
                              0x00,0x00,0x18,0x1C,0xFE,0xFE,0x00,0x00,0x00,0x00,  // 1
                              0x00,0x00,0x60,0x60,0x7F,0x7F,0x60,0x60,0x00,0x00,
                              0x18,0x1C,0x0E,0x06,0x06,0x06,0x86,0xCE,0xFC,0x78,  // 2
                              0x60,0x70,0x78,0x7C,0x6E,0x67,0x63,0x61,0x60,0x60,
                              0x00,0x0C,0x8E,0x86,0x86,0x86,0x86,0xCE,0xFC,0x78,  // 3
                              0x18,0x38,0x71,0x61,0x61,0x61,0x61,0x73,0x3F,0x1E,
                              0x80,0xC0,0xE0,0x70,0x38,0x1C,0xFE,0xFE,0x00,0x00,  // 4
                              0x07,0x07,0x06,0x06,0x06,0x06,0x7F,0x7F,0x06,0x06,
                              0x7E,0x7E,0x66,0x66,0x66,0x66,0x66,0xE6,0xC6,0x86,  // 5
                              0x18,0x38,0x70,0x60,0x60,0x60,0x60,0x70,0x3F,0x1F,
                              0xF8,0xFC,0x8E,0x86,0x86,0x86,0x86,0x8E,0x0C,0x00,  // 6
                              0x1F,0x3F,0x71,0x61,0x61,0x61,0x61,0x73,0x3F,0x1E,
                              0x1E,0x1E,0x06,0x06,0x06,0x06,0x86,0xC6,0xFE,0x7E,  // 7
                              0x00,0x00,0x00,0x00,0x7E,0x7F,0x03,0x01,0x00,0x00,
                              0x00,0xB8,0xFC,0xCE,0x86,0x86,0xCE,0xFC,0xB8,0x00,  // 8
                              0x1F,0x3F,0x73,0x61,0x61,0x61,0x61,0x73,0x3F,0x1F,
                              0x78,0xFC,0xCE,0x86,0x86,0x86,0x86,0x8E,0xFC,0xF8,  // 9
                              0x00,0x30,0x71,0x61,0x61,0x61,0x61,0x71,0x3F,0x1F,
                              0x00,0x00,0x00,0x30,0x78,0x78,0x30,0x00,0x00,0x00,  // :
                              0x00,0x00,0x0C,0x1E,0x1E,0x0C,0x00,0x00,0x00,0x00,
                              0x00,0x00,0x30,0x78,0x78,0x30,0x00,0x00,0x00,0x00,  // ;
                              0x00,0x00,0x66,0x76,0x3E,0x1E,0x00,0x00,0x00,0x00,
                              0x80,0xC0,0xE0,0x70,0x38,0x1C,0x0E,0x06,0x00,0x00,  // <
                              0x01,0x03,0x07,0x0E,0x1C,0x38,0x70,0x60,0x00,0x00,
                              0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,  // =
                              0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
                              0x00,0x00,0x06,0x0E,0x1C,0x38,0x70,0xE0,0xC0,0x80,  // >
                              0x00,0x00,0x60,0x70,0x38,0x1C,0x0E,0x07,0x03,0x01,
                              0x18,0x1C,0x0E,0x06,0x06,0x06,0x86,0xCE,0xFC,0x78,  // ?
                              0x00,0x00,0x00,0x00,0x6E,0x6F,0x03,0x01,0x00,0x00,
                              
                              0x18,0x1C,0x8E,0x86,0x86,0x86,0x06,0x0E,0xFC,0xF8,  // @
                              0x1E,0x3F,0x73,0x61,0x7F,0x7F,0x60,0x70,0x3F,0x1F,
                              0xF8,0xFC,0x0E,0x06,0x06,0x06,0x06,0x0E,0xFC,0xF8,  // A
                              0x7F,0x7F,0x03,0x03,0x03,0x03,0x03,0x03,0x7F,0x7F,
                              0xFE,0xFE,0x86,0x86,0x86,0x86,0xCE,0xFC,0x78,0x00,  // B
                              0x7F,0x7F,0x61,0x61,0x61,0x61,0x61,0x73,0x3F,0x1E,
                              0xF8,0xFC,0x0E,0x06,0x06,0x06,0x06,0x0E,0x1C,0x18,  // C
                              0x1F,0x3F,0x70,0x60,0x60,0x60,0x60,0x70,0x38,0x18,
                              0xFE,0xFE,0x06,0x06,0x06,0x06,0x06,0x0E,0xFC,0xF8,  // D
                              0x7F,0x7F,0x60,0x60,0x60,0x60,0x60,0x70,0x3F,0x1F,
                              0xFE,0xFE,0x86,0x86,0x86,0x86,0x86,0x86,0x06,0x06,  // E
                              0x7F,0x7F,0x61,0x61,0x61,0x61,0x61,0x61,0x60,0x60,
                              0xFE,0xFE,0x86,0x86,0x86,0x86,0x86,0x86,0x06,0x06,  // F
                              0x7F,0x7F,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,
                              0xF8,0xFC,0x0E,0x06,0x86,0x86,0x86,0x8E,0x9C,0x98,  // G
                              0x1F,0x3F,0x70,0x60,0x61,0x61,0x61,0x71,0x3F,0x1F,
                              0xFE,0xFE,0x80,0x80,0x80,0x80,0x80,0x80,0xFE,0xFE,  // H
                              0x7F,0x7F,0x01,0x01,0x01,0x01,0x01,0x01,0x7F,0x7F,
                              0x00,0x00,0x06,0x06,0xFE,0xFE,0x06,0x06,0x00,0x00,  // I
                              0x00,0x00,0x60,0x60,0x7F,0x7F,0x60,0x60,0x00,0x00,
                              0x00,0x00,0x00,0x00,0x06,0x06,0xFE,0xFE,0x06,0x06,  // J
                              0x18,0x38,0x70,0x60,0x60,0x70,0x3F,0x1F,0x00,0x00,
                              0xFE,0xFE,0x80,0xC0,0xE0,0x70,0x38,0x1C,0x0E,0x06,  // K
                              0x7F,0x7F,0x01,0x03,0x07,0x0E,0x1C,0x38,0x70,0x60,
                              0xFE,0xFE,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,  // L
                              0x7F,0x7F,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,
                              0xFE,0xFE,0x1C,0x38,0xF0,0xF0,0x38,0x1C,0xFE,0xFE,  // M
                              0x7F,0x7F,0x00,0x00,0x01,0x01,0x00,0x00,0x7F,0x7F,
                              0xFE,0xFE,0x70,0xE0,0xC0,0x80,0x00,0x00,0xFE,0xFE,  // N
                              0x7F,0x7F,0x00,0x00,0x01,0x03,0x07,0x0E,0x7F,0x7F,
                              0xF8,0xFC,0x0E,0x06,0x06,0x06,0x06,0x0E,0xFC,0xF8,  // O
                              0x1F,0x3F,0x70,0x60,0x60,0x60,0x60,0x70,0x3F,0x1F,

                              0xFE,0xFE,0x86,0x86,0x86,0x86,0x86,0xCE,0xFC,0x78,  // P
                              0x7F,0x7F,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,
                              0xF8,0xFC,0x0E,0x06,0x06,0x06,0x06,0x0E,0xFC,0xF8,  // Q
                              0x1F,0x3F,0x70,0x60,0x66,0x6E,0x7C,0x38,0x7F,0x6F,
                              0xFE,0xFE,0x86,0x86,0x86,0x86,0x86,0xCE,0xFC,0x78,  // R
                              0x7F,0x7F,0x01,0x03,0x07,0x0F,0x1D,0x39,0x70,0x60,
                              0x78,0xFC,0xCE,0x86,0x86,0x86,0x86,0x8E,0x1C,0x18,  // S
                              0x18,0x38,0x71,0x61,0x61,0x61,0x61,0x73,0x3F,0x1E,
                              0x06,0x06,0x06,0x06,0xFE,0xFE,0x06,0x06,0x06,0x06,  // T
                              0x00,0x00,0x00,0x00,0x7F,0x7F,0x00,0x00,0x00,0x00,
                              0xFE,0xFE,0x00,0x00,0x00,0x00,0x00,0x00,0xFE,0xFE,  // U
                              0x1F,0x3F,0x70,0x60,0x60,0x60,0x60,0x70,0x3F,0x1F,
                              0xFE,0xFE,0x00,0x00,0x00,0x00,0x00,0x00,0xFE,0xFE,  // V
                              0x07,0x0F,0x1C,0x38,0x70,0x70,0x38,0x1C,0x0F,0x07,
                              0xFE,0xFE,0x00,0x00,0xC0,0xC0,0x00,0x00,0xFE,0xFE,  // W
                              0x7F,0x7F,0x38,0x1C,0x0F,0x0F,0x1C,0x38,0x7F,0x7F,
                              0x1E,0x3E,0x70,0xE0,0xC0,0xC0,0xE0,0x70,0x3E,0x1E,  // X
                              0x78,0x7C,0x0E,0x07,0x03,0x03,0x07,0x0E,0x7C,0x78,
                              0x7E,0xFE,0xC0,0x80,0x00,0x00,0x80,0xC0,0xFE,0x7E,  // Y
                              0x00,0x00,0x01,0x03,0x7F,0x7F,0x03,0x01,0x00,0x00,
                              0x06,0x06,0x06,0x06,0x86,0xC6,0xE6,0x76,0x3E,0x1E,  // Z
                              0x78,0x7C,0x6E,0x67,0x63,0x61,0x60,0x60,0x60,0x60,
                              0x00,0x00,0xFE,0xFE,0x06,0x06,0x06,0x06,0x00,0x00,  // [
                              0x00,0x00,0x7F,0x7F,0x60,0x60,0x60,0x60,0x00,0x00,
                              
                              0x0C,0x1C,0x38,0x70,0xE0,0xC0,0x80,0x00,0x00,0x00,  // \
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,  //BUG!!! have to insert these dummy bytes or rest of characters are incorrectly offset
                              0x00,0x00,0x00,0x00,0x00,0x01,0x03,0x07,0x0E,0x0C,
                              0x00,0x00,0x06,0x06,0x06,0x06,0xFE,0xFE,0x00,0x00,  // ]
                              0x00,0x00,0x60,0x60,0x60,0x60,0x7F,0x7F,0x00,0x00,
                              0x60,0x70,0x38,0x1C,0x0E,0x0E,0x1C,0x38,0x70,0x60,  // ^
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,  // _
                              0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,

                              0x00,0x00,0x06,0x0E,0x1C,0x38,0x70,0x60,0x00,0x00,  // `
                              0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                              0x00,0x00,0x60,0x60,0x60,0x60,0x60,0xE0,0xC0,0x80,  // a
                              0x18,0x3C,0x7E,0x66,0x66,0x66,0x66,0x66,0x7F,0x7F,
                              0xFE,0xFE,0x80,0xC0,0xE0,0x60,0x60,0xE0,0xC0,0x80,  // b
                              0x7F,0x7F,0x63,0x61,0x60,0x60,0x60,0x70,0x3F,0x1F,
                              0x80,0xC0,0xE0,0x60,0x60,0x60,0x60,0x60,0x00,0x00,  // c
                              0x1F,0x3F,0x70,0x60,0x60,0x60,0x60,0x70,0x38,0x18,
                              0x80,0xC0,0xE0,0x60,0x60,0xE0,0xC0,0x80,0xFE,0xFE,  // d
                              0x1F,0x3F,0x70,0x60,0x60,0x60,0x61,0x63,0x7F,0x7F,
                              0x80,0xC0,0xE0,0x60,0x60,0x60,0x60,0xE0,0xC0,0x80,  // e
                              0x1F,0x3F,0x76,0x66,0x66,0x66,0x66,0x66,0x07,0x07,
                              0x80,0x80,0xF8,0xFC,0x8E,0x86,0x06,0x0E,0x1C,0x18,  // f
                              0x01,0x01,0x7F,0x7F,0x01,0x01,0x00,0x00,0x00,0x00,
                              0xE0,0xF0,0x38,0x18,0x18,0x18,0x18,0x18,0xF8,0xF8,  // g
                              0x01,0x03,0x67,0x66,0x66,0x66,0x66,0x76,0x3F,0x1F,
                              0xFE,0xFE,0x80,0xC0,0xE0,0x60,0x60,0xE0,0xC0,0x80,  // h
                              0x7F,0x7F,0x03,0x01,0x00,0x00,0x00,0x00,0x7F,0x7F,
                              0x00,0x00,0x60,0x60,0xE6,0xE6,0x00,0x00,0x00,0x00,  // i
                              0x00,0x00,0x60,0x60,0x7F,0x7F,0x60,0x60,0x00,0x00,
                              0x00,0x00,0x00,0x00,0x60,0x60,0xE6,0xE6,0x00,0x00,  // j
                              0x18,0x38,0x70,0x60,0x60,0x70,0x3F,0x1F,0x00,0x00,
                              0xFE,0xFE,0x00,0x00,0x80,0xC0,0xE0,0x60,0x00,0x00,  // k
                              0x7F,0x7F,0x07,0x0F,0x1F,0x39,0x70,0x60,0x00,0x00,
                              0x00,0x00,0x06,0x06,0xFE,0xFE,0x00,0x00,0x00,0x00,  // l
                              0x00,0x00,0x60,0x60,0x7F,0x7F,0x60,0x60,0x00,0x00,
                              0xE0,0xE0,0x60,0xE0,0xC0,0xE0,0x60,0xE0,0xC0,0x80,  // m
                              0x7F,0x7F,0x00,0x00,0x07,0x07,0x00,0x00,0x7F,0x7F,
                              0xE0,0xE0,0x80,0xC0,0xE0,0x60,0x60,0xE0,0xC0,0x80,  // n
                              0x7F,0x7F,0x03,0x01,0x00,0x00,0x00,0x00,0x7F,0x7F,
                              0x80,0xC0,0xE0,0x60,0x60,0x60,0x60,0xE0,0xC0,0x80,  // o
                              0x1F,0x3F,0x70,0x60,0x60,0x60,0x60,0x70,0x3F,0x1F,
                              
                              0xE0,0xE0,0x60,0x60,0x60,0x60,0x60,0xE0,0xC0,0x80,  // p
                              0x7F,0x7F,0x06,0x06,0x06,0x06,0x06,0x07,0x03,0x01,
                              0x80,0xC0,0xE0,0x60,0x60,0xE0,0xC0,0x80,0xE0,0xE0,  // q
                              0x01,0x03,0x07,0x06,0x06,0x06,0x07,0x07,0x7F,0x7F,
                              0xE0,0xE0,0x80,0xC0,0xE0,0x60,0x60,0xE0,0xC0,0x80,  // r
                              0x7F,0x7F,0x03,0x01,0x00,0x00,0x00,0x00,0x01,0x01,
                              0x80,0xC0,0xE0,0x60,0x60,0x60,0x60,0x60,0xE0,0xC0,  // s
                              0x31,0x73,0x67,0x66,0x66,0x66,0x66,0x7E,0x3C,0x18,
                              0x60,0x60,0xFE,0xFE,0x60,0x60,0x00,0x00,0x00,0x00,  // t
                              0x00,0x00,0x1F,0x3F,0x70,0x60,0x60,0x70,0x38,0x18,
                              0xE0,0xE0,0x00,0x00,0x00,0x00,0x00,0x00,0xE0,0xE0,  // u
                              0x1F,0x3F,0x70,0x60,0x60,0x70,0x38,0x1C,0x7F,0x7F,
                              0xE0,0xE0,0x00,0x00,0x00,0x00,0x00,0x00,0xE0,0xE0,  // v
                              0x07,0x0F,0x1C,0x38,0x70,0x70,0x38,0x1C,0x0F,0x07,
                              0xE0,0xE0,0x00,0x00,0x80,0x80,0x00,0x00,0xE0,0xE0,  // w
                              0x1F,0x3F,0x78,0x70,0x3F,0x3F,0x70,0x78,0x3F,0x1F,
                              0x60,0xE0,0xC0,0x80,0x00,0x00,0x80,0xC0,0xE0,0x60,  // x
                              0x60,0x70,0x39,0x1F,0x0F,0x0F,0x1F,0x39,0x70,0x60,
                              0xE0,0xE0,0x00,0x00,0x00,0x00,0x00,0x00,0xE0,0xE0,  // y
                              0x01,0x03,0x67,0x66,0x66,0x66,0x66,0x76,0x3F,0x1F,
                              0x60,0x60,0x60,0x60,0x60,0x60,0xE0,0xE0,0xE0,0x60,  // z
                              0x60,0x70,0x78,0x7C,0x6E,0x67,0x63,0x61,0x60,0x60,
                              0x00,0x00,0x80,0xC0,0xF8,0x7C,0x0E,0x06,0x00,0x00,  // {
                              0x00,0x00,0x01,0x03,0x1F,0x3E,0x70,0x60,0x00,0x00,
                              0x00,0x00,0x00,0x00,0xFE,0xFE,0x00,0x00,0x00,0x00,  // |
                              0x00,0x00,0x00,0x00,0x7F,0x7F,0x00,0x00,0x00,0x00,
                              0x00,0x00,0x06,0x0E,0x7C,0xF8,0xC0,0x80,0x00,0x00,  // }
                              0x00,0x00,0x60,0x70,0x3E,0x1F,0x03,0x01,0x00,0x00,
                              0x80,0x80,0x80,0x80,0x98,0xB8,0xF0,0xE0,0xC0,0x80,  // ->
                              0x01,0x01,0x01,0x01,0x19,0x1D,0x0F,0x07,0x03,0x01,
                              0x80,0xC0,0xE0,0xF0,0xB8,0x98,0x80,0x80,0x80,0x80,  // <-
                              0x01,0x03,0x07,0x0F,0x1D,0x19,0x01,0x01,0x01,0x01};

const uint8_t Icons[] PROGMEM = {32,32, //32x32 pixels fan+filter icons
      0xF0,0xF8,0xFC,0x3E,0x1F,0x0F,0x07,0x07,0x03,0x03,0x03,0x03,0x03,0x03,0xE3,0xF3,0xFB,0xFB,0xFB,0xFB,0xFB,0x7B,0x33,0x13,0x07,0x07,0x0F,0x1F,0x3E,0xFC,0xF8,0xF0,  //Fan0
      0xFF,0xFF,0x00,0xFC,0xFF,0xFE,0xFC,0xF8,0xF0,0xC0,0x00,0xC0,0xE0,0xF0,0xFB,0xFB,0xFB,0xFB,0xF1,0xE1,0xC0,0x00,0xC0,0xC0,0xC0,0xC0,0xC0,0x80,0x00,0x00,0xFF,0xFF,
      0xFF,0xFF,0x00,0x00,0x01,0x03,0x03,0x03,0x03,0x03,0x00,0x03,0x87,0x8F,0xDF,0xDF,0xDF,0xDF,0x0F,0x07,0x03,0x00,0x03,0x0F,0x1F,0x3F,0x7F,0xFF,0x3F,0x00,0xFF,0xFF,
      0x0F,0x1F,0x3F,0x7C,0xF8,0xF0,0xE0,0xE0,0xC8,0xCC,0xDE,0xDF,0xDF,0xDF,0xDF,0xDF,0xCF,0xC7,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xE0,0xE0,0xF0,0xF8,0x7C,0x3F,0x1F,0x0F,
      0xF0,0xF8,0xFC,0x3E,0x1F,0x0F,0xC7,0xE7,0xF3,0xF3,0xFB,0xFB,0xFB,0x0B,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x83,0xC7,0xC7,0x8F,0x1F,0x3E,0xFC,0xF8,0xF0,  //Fan1
      0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x01,0x03,0x07,0x0F,0xCF,0xE7,0xF0,0xF8,0xF8,0xF8,0xF8,0xF0,0xE0,0xCC,0x1E,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x3C,0x00,0xFF,0xFF,
      0xFF,0xFF,0x00,0x3C,0xF8,0xF8,0xF8,0xF8,0xF8,0xF8,0x78,0x33,0x07,0x0F,0x1F,0x1F,0x1F,0x1F,0x0F,0xE7,0xF3,0xF0,0xE0,0xC0,0x80,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,
      0x0F,0x1F,0x3F,0x7C,0xF8,0xF1,0xE3,0xE3,0xC1,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xD0,0xDF,0xDF,0xDF,0xCF,0xCF,0xE7,0xE3,0xF0,0xF8,0x7C,0x3F,0x1F,0x0F,
      0xF0,0xF8,0xFC,0x3E,0x1F,0x0F,0x07,0x07,0xF3,0xFB,0xFB,0xFB,0x7B,0x7B,0x7B,0x7B,0x7B,0x7B,0x7B,0x7B,0x7B,0x7B,0x7B,0x33,0x07,0x07,0x0F,0x1F,0x3E,0xFC,0xF8,0xF0,  //Replace Filter
      0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,
      0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,
      0x0F,0x1F,0x3F,0x7C,0xF8,0xF0,0xE0,0xE0,0xCF,0xDF,0xDF,0xCF,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xE0,0xE0,0xF0,0xF8,0x7C,0x3F,0x1F,0x0F};

const uint8_t Battery[] PROGMEM = {29,16,  //29x16 pixels battery icons
      /*0xF0,0x10,0x1E,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFE, //Battery 4
      0x0F,0x08,0x78,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFE, //Battery 3
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFE, //Battery 2
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xFA,0xFA,0xFA,0xFA,0xFA,0x02,0xFE, //Battery 1
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x5F,0x5F,0x5F,0x5F,0x5F,0x40,0x7F,*/
      0xF0,0x10,0x1E,0x02,0xFA,0xE2,0x9A,0x7A,0xFA,0xFA,0xFA,0xFA,0xFA,0xE2,0x9A,0x7A,0xFA,0xFA,0xFA,0xFA,0xFA,0xE2,0x9A,0x7A,0xFA,0xFA,0xFA,0x02,0xFE,  //Battery 4
      0x0F,0x08,0x78,0x40,0x5F,0x5F,0x5F,0x5E,0x59,0x47,0x5F,0x5F,0x5F,0x5F,0x5F,0x5E,0x59,0x47,0x5F,0x5F,0x5F,0x5F,0x5F,0x5E,0x59,0x47,0x5F,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x1A,0x7A,0xFA,0xFA,0xFA,0xFA,0xFA,0xE2,0x9A,0x7A,0xFA,0xFA,0xFA,0xFA,0xFA,0xE2,0x9A,0x7A,0xFA,0xFA,0xFA,0x02,0xFE,  //Battery 3
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x41,0x47,0x5F,0x5F,0x5F,0x5F,0x5F,0x5E,0x59,0x47,0x5F,0x5F,0x5F,0x5F,0x5F,0x5E,0x59,0x47,0x5F,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x1A,0x7A,0xFA,0xFA,0xFA,0xFA,0xFA,0xE2,0x9A,0x7A,0xFA,0xFA,0xFA,0x02,0xFE,  //Battery 2
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x41,0x47,0x5F,0x5F,0x5F,0x5F,0x5F,0x5E,0x59,0x47,0x5F,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x1A,0x7A,0xFA,0xFA,0xFA,0x02,0xFE,  //Battery 1
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x41,0x47,0x5F,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xF2,0xFA,0xFA,0xF2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xFE, //Exclamation
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x5B,0x5B,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x0A,0x1A,0x32,0x62,0xC2,0x82,0xC2,0x62,0x32,0x1A,0x0A,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xFE, //X
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x50,0x58,0x4C,0x46,0x43,0x41,0x43,0x46,0x4C,0x58,0x50,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x82,0x82,0x82,0x82,0x82,0xE2,0xF2,0xF2,0xF2,0xFA,0xFA,0xFA,0x62,0x62,0x62,0x02,0x02,0x02,0x02,0x02,0x02,0xFE, //Charge
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x41,0x41,0x41,0x41,0x41,0x47,0x4F,0x4F,0x4F,0x5F,0x5F,0x5F,0x46,0x46,0x46,0x40,0x40,0x40,0x40,0x40,0x40,0x7F,
      0xF0,0x10,0x1E,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x22,0x72,0xFA,0xFA,0xFA,0xBA,0x1A,0x0A,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xFE, //Lightning Bolt
      0x0F,0x08,0x78,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x58,0x4D,0x47,0x43,0x41,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x7F,
      };

const uint8_t logo[] PROGMEM = {32,32,  //32x32 , 128 bytes sjm4306 logo
                                0x00, 0x80, 0x80, 0xE0, 0xE0, 0x18, 0x18, 0x1E, 0x1E, 0x18, 0x18, 0x1E, 0x1E,
                                0x18, 0x18, 0x1E, 0x1E, 0x18, 0x18, 0x9E, 0x9E, 0x98, 0x98, 0x9E, 0x9E, 0x18,
                                0x18, 0xE0, 0xE0, 0x80, 0x80, 0x00, 0x00, 0x99, 0x99, 0xFF, 0xFF, 0x00, 0x00,
                                0x00, 0x00, 0x00, 0x00, 0xFE, 0xFE, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86, 0x9F,
                                0x9F, 0x19, 0x19, 0x1F, 0x1F, 0x00, 0x00, 0xFF, 0xFF, 0x99, 0x99, 0x00, 0x00,
                                0x99, 0x99, 0xFF, 0xFF, 0x00, 0x00, 0xF8, 0xF8, 0x98, 0x98, 0xF9, 0xF9, 0x61,
                                0x61, 0x61, 0x61, 0x61, 0x61, 0x7F, 0x7F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                                0xFF, 0xFF, 0x99, 0x99, 0x00, 0x00, 0x01, 0x01, 0x07, 0x07, 0x18, 0x18, 0x79,
                                0x79, 0x19, 0x19, 0x79, 0x79, 0x18, 0x18, 0x78, 0x78, 0x18, 0x18, 0x78, 0x78,
                                0x18, 0x18, 0x78, 0x78, 0x18, 0x18, 0x07, 0x07, 0x01, 0x01, 0x00
                               };

void writeI2C(unsigned char devaddr, unsigned char reg, bool b, unsigned char data0, unsigned char data1){  //send 1 or 2 I2C bytes to a specified device address
  Wire.beginTransmission(devaddr); //Write
  Wire.write(reg);       //Word Address
  Wire.write(data0);     //Data
  if(b){Wire.write(data1);}
  Wire.endTransmission(); //Send bytes and stop transmitting
  //delayMicroseconds(10);
}

void disp_init(void){ //power cycle oled to reset it and reinitialize it (including clearing gddram)
  digitalWrite(RST,LOW);
  delay(50);
  digitalWrite(RST,HIGH);
  delay(50);

  Wire.begin();
  Wire.setClock(400000);  //100kHz
  
  writeI2C(OLEDaddr,CMD,0,0xAE,0);    //Set display off
  writeI2C(OLEDaddr,CMD,1,0xD5,0x80); //Set display clock divide ratio, Set display oscillator freq
  writeI2C(OLEDaddr,CMD,1,0xA8,0x1F); //Set multiplex ratio
  writeI2C(OLEDaddr,CMD,1,0xD3,0x00); //Set display offset
  writeI2C(OLEDaddr,CMD,0,0x40,0);    //Set display start line
  writeI2C(OLEDaddr,CMD,1,0x8D,0x14); //Set charge pump
  writeI2C(OLEDaddr,CMD,0,0xA1,0);    //Set segment remap
  writeI2C(OLEDaddr,CMD,0,0xC8,0);    //Set com output scan dir
  writeI2C(OLEDaddr,CMD,1,0xDA,0x02); //Set com pins hardware config
  writeI2C(OLEDaddr,CMD,1,0x81,0x8F); //Set contrast control
  writeI2C(OLEDaddr,CMD,1,0xD9,0xF1); //Set precharge period
  writeI2C(OLEDaddr,CMD,1,0xDB,0x40); //Set vcomh deselect level
  writeI2C(OLEDaddr,CMD,0,0x2E,0);    //Deactivate Scrolling
  writeI2C(OLEDaddr,CMD,0,0x10,0);    //Set Higher Column Start Address
  writeI2C(OLEDaddr,CMD,0,0x00,0);    //Set Lower Column Start Address
  writeI2C(OLEDaddr,CMD,0,0xA4,0);    //Set entire display on/off
  writeI2C(OLEDaddr,CMD,0,0xA6,0);    //Set normal/inverse display
  writeI2C(OLEDaddr,CMD,1,0x20,0x00); //Set memory addressing mode
  clr();
  writeI2C(OLEDaddr,CMD,0,0xAF,0x00); //Set display on
}

void set_xy(char x, char y){  //move X,Y cursor location to write image data (0<=X<=127, 0<=Y<=7)
  Wire.beginTransmission(OLEDaddr);
  Wire.write(CMD);
  Wire.write(0x21); //set x
  Wire.write(0x00+x);
  Wire.write(0x7F);
  Wire.endTransmission();
  delayMicroseconds(10);
  writeI2C(OLEDaddr,CMD,0,0xB0+y,0); //set y
}

void clr(void){ //clear entire screen by writing 0 to all pixels stored in gddram, and reset cursor to 0,0
  set_xy(0,0);
  for(int i=0;i<8192;i++){
    writeI2C(OLEDaddr,DAT,0,0x00,0);
  }
  set_xy(0,0);
}

void printchar(char x, char y, char c, int font, bool inv){ //writes a single character to location X,Y on screen, can specify font and inversion
  unsigned char w,h=0;
  w = pgm_read_byte_near(font);
  h = pgm_read_byte_near(font + 1);
  for(int j=0;j<h/8;j++){
    set_xy(x,y+j);
    for(int i=0;i<w;i++){
      writeI2C(OLEDaddr,0b01000000,0,pgm_read_byte_near(font+w*j+2+(w*h/8)*(c-32)+i)^0xFF*inv,0);
    }
    writeI2C(OLEDaddr,DAT,0,0xFF*inv,0);
  }
}

void writeString(char x, char y, String str, int font, bool inv){ //similar to printchar, but writes a whole string to the screen
  unsigned char len = str.length();
  unsigned char w = pgm_read_byte_near(font);
  unsigned char p=0;
  while(p<len){
    printchar(x+p*(w+1),y,str.charAt(p),font,inv);
    p++;
  }
}

void drawBMP(int x, int y, unsigned long bmp, unsigned char elem, bool inv){  //write bitmap image to location X,Y on screen, can specify array bmp and elem index of image
  unsigned char w=pgm_read_byte(bmp);
  unsigned char h=pgm_read_byte(bmp+1);
  for(int j=0;j<h/8;j++){
    set_xy(x,y+j);
    for(int i=0;i<w;i++){
      writeI2C(OLEDaddr,DAT,0,pgm_read_byte_near(bmp+2+j*w+i+w*(h/8)*elem)^(inv*0xFF),0);
    }
  }
}

unsigned long vbat(void){ //returns measured supply voltage in mV (requires diode resistor reference circuitry which outputs ~0.7V)
  unsigned long val=0;
  /*ADMUX=0b01001110; //AVcc reference, measure internal 1.1V reference
  //ADCSRB=0b00000000;  //free running mode
  delayMicroseconds(500);
  ADCSRA=0b11000000;  //turn on ADC, x2 clock prescale, start conversion
  while(ADCSRA&(1<<ADSC)); //wait for conversion to finish
  //val=(ADCH<<8)|ADCL;
  return (1100UL*1023/ADC); //calculate value in mV*/

  pinMode(Vref,OUTPUT);
  digitalWrite(Vref,HIGH);
  delay(100);
  val=analogRead(Vdiode);
  digitalWrite(Vref,LOW);
  pinMode(Vref,INPUT);
  val=534883/val;    //(unsigned long) (1023-(716100/(double) val));  716100=2^10*699mV, measured 778240=2^10*760mV, 3.62V supply (measures as 4.25V) so ref is actually 647mV!
  return val;
}

void bar(int x, int y, int s){  //draws a small 16 pixel tall bar graph with 0-10 bars at location X,Y on the screen
  for(int i=1;i>=0;i--){
    unsigned int a,b,c=0;
    b=0b1100000000000000;
    for(int j=0;j<s;j++){
      set_xy(x+3*j,y+i);
      b|=b>>1|0b1000000000000000;
      a=b&0x00FF;
      c=b>>8;
      writeI2C(OLEDaddr,DAT,0,(i==0)*a+(i==1)*c,0);
      writeI2C(OLEDaddr,DAT,0,(i==0)*a+(i==1)*c,0);
    }
  }
  for(int i=0;i<(10-s);i++){
    set_xy(x+27-3*i,y);
    writeI2C(OLEDaddr,DAT,0,0,0);
    writeI2C(OLEDaddr,DAT,0,0,0);
    set_xy(x+27-3*i,y+1);
    writeI2C(OLEDaddr,DAT,0,0b10000000,0);
    writeI2C(OLEDaddr,DAT,0,0b10000000,0);
  }
}

void brightness(unsigned char brt){ //set oled screen brightness from 0-255
  writeI2C(OLEDaddr,CMD,1,0x81,brt); //Set contrast control
}

void init_PWM(void){  //initialize PWM for D9/OC1A (Timer1), 62.5 kHz, 7 bit resolution
  //Dont mess with Timer0 which is used for delay timing
  TCCR1A = _BV(COM1A1) | _BV(WGM12) | _BV(WGM11);  //7 bit (set TOP below) pwm mode 3 on OC1A pin (OC1B pin is regular IO)
  TCCR1B = _BV(CS10) | _BV(WGM13); //prescale by 1
  OCR1A = 0;  //initialize to 0% duty
  ICR1L = 128;  //set TOP of timer1 for 7 bit resolution or the frequency would halve
}

void init_gpio(void){ //initialize all gpio
  pinMode(LED,OUTPUT);
  pinMode(SCL,OUTPUT);
  pinMode(SDA,OUTPUT);
  pinMode(RST,OUTPUT);
  pinMode(PWM,OUTPUT);
  pinMode(Vboost,OUTPUT);

  pinMode(Vref,INPUT);
  pinMode(Vdiode,INPUT);
  pinMode(PWR,INPUT_PULLUP);
  pinMode(B1,INPUT_PULLUP);
  pinMode(B2,INPUT_PULLUP);
  pinMode(CHG,INPUT_PULLUP);

  digitalWrite(SCL,LOW);
  digitalWrite(SDA,LOW);
  digitalWrite(RST,HIGH);
  digitalWrite(Vboost,LOW);
}

void splashscreen(void){
  if(1){
    drawBMP(0,0,logo,0,NORM);
    writeString(33,0,"FumeFan",BigFont,NORM);
    writeString(33,2,"v1.0",SmallFont,NORM);
    writeString(33,3,"sjm4306 2021",SmallFont,NORM);
    delay(2000);
  }
  clr();
}

void setup() {
  init_gpio();
  init_PWM();
  
  disp_init();
  Serial.begin(9600); //initialize uart serial port
  //brightness(50);

  //EEPROM.update(4,0b10001001);  //639h59m50s, force total time count to test filter replace alarm
  //EEPROM.update(3,0b01010100);
  //EEPROM.update(2,0b00011000);
  //EEPROM.update(1,0b11110000);

  //Restore EEPROM backed values
  Speed=EEPROM.read(0);
  if(Speed>10){Speed=5;EEPROM.update(0,5);}
  //totaltime=(unsigned long) EEPROM.read(1)|(unsigned long) EEPROM.read(2)<<8|(unsigned long) EEPROM.read(3)<<16|(unsigned long) EEPROM.read(4)<<24;  //totaltime is 32 bit unsigned long so needs 4 bytes from eeprom
  
  splashscreen();
}

void loop() {  
  unsigned long cnt=0;
  long tme=0;
  char v[4];  //for itoa 

  bool Toggle=0;
  int volt=0;
  int hours=0,minutes=0,seconds=0,offset=0;
  unsigned long t=0;
  unsigned char b=0;
  
  while(1){ //main loop
    unsigned int dim=0;
    
    //Charging while "off" here! (Important, charging while on will shut off fan and then start charging)
    while(!digitalRead(CHG)){
      if(dim<10){ //display slightly dims after a few seconds to save the display
        writeI2C(OLEDaddr,CMD,1,0x81,0x8F);  //Set brightness
      }else{
        writeI2C(OLEDaddr,CMD,1,0x81,0x00);  //dim
      }
      digitalWrite(Vboost,LOW);
      digitalWrite(PWM,LOW);
      if(sleep_flg==0){clr();}
      drawBMP(50,1,Battery,b%4,NORM);
      b--;
      dim++;
      delay(1000);
      sleep_flg=1;
    }
    writeI2C(OLEDaddr,CMD,1,0x81,0x8F); //Set brightness
    
    if(!digitalRead(PWR)||sleep_flg==1){  //if power button is pressed or sleep flag request, turn off device
      digitalWrite(Vboost,LOW); //turn off 12v boost converter for fan when shutting off
      sleep_flg=0;
      digitalWrite(LED,HIGH);
      delay(250);
      digitalWrite(LED,LOW);
      //save variables to EEPROM before shutting off, only update changed values to save memory from excessive wear
      EEPROM.update(0,Speed);
      //EEPROM.update(1,(unsigned char) (t&0x000000FF));
      //EEPROM.update(2,(unsigned char) (t>>8)&0x000000FF);
      //EEPROM.update(3,(unsigned char) (t>>16)&0x000000FF);
      //EEPROM.update(4,(unsigned char) (t>>24)&0x000000FF);

      attachInterrupt(0,wakeInterrupt,FALLING); //enable interrupts for power button and charge indicator inputs
      attachInterrupt(1,chargeInterrupt,FALLING);
      
      set_sleep_mode(SLEEP_MODE_PWR_DOWN);  //power down mode with all peripherals shut off to minimize power draw during sleep
      pinMode(0,INPUT);   //set all pins except pin change interrupt sources to inputs to minimize power draw during sleep
      digitalWrite(0,LOW);
      pinMode(1,INPUT);
      digitalWrite(1,LOW);
      for(int i=4;i<18;i++){
        pinMode(i,OUTPUT);
        digitalWrite(i,LOW);
      }
      //sleep_bod_disable();
      wdt_disable();       //disable watchdog
      ADCSRA &= ~(1<<7);   //ADEN=0, turn off adc
      PRR=0b11101111;      //power down individual peripherals to lower current consumption
      MCUCR |= (3<<5);     //set both BODS and BODSE at the same time
      MCUCR = (MCUCR & ~(1<<5))|(1<<6); //then set BODS bit and clear BODSE bit at the same time

      //clr();
      
      sleep_enable();
      sleep_mode();
      sleep_disable();  //wake up here when power button pressed
      PRR=0b00000000;   //reinitialize everything
      ADCSRA|=1<<7;     //ADEN=1, turn adc back on!!!
      init_gpio();
      disp_init();
      init_PWM();
      //delay(500);
      splashscreen();
      clr();
      if(!digitalRead(B1)){ //Hold B1 while turning on (through splashscreen) to reset filter time
        totaltime=t=tme=0;
      }
      digitalWrite(Vboost,HIGH);  //turn on 12v boost converter for fan when powering on
    }

    if(!digitalRead(B1)){if(Speed<10)Speed++;bar(33,2,Speed);delay(250);} //increment/decrement fan speed with buttons
    if(!digitalRead(B2)){if(Speed>1)Speed--;bar(33,2,Speed);delay(250);}
    bar(33,2,Speed);
    OCR1A=fan_speed[Speed-1];

    if(cnt%10==0){  //execute this block every second
      tme+=1000;    //keep track of millisecond count
      //digitalWrite(LED,Toggle);
      Toggle^=1;
      if(hours<640){          //Draw Fan Animation (or display filter icon if more than 640 hours have elapsed since last replacement)
        drawBMP(0,0,Icons,Toggle,NORM);
      }else{
        drawBMP(0,0,Icons,Filter-Toggle,NORM);
      }

      volt=vbat();  //TEST BATTERY VOLTAGE MEASUREMENT!!!!!!!!!!!!!!!!!!!!!!!
      
      writeString(74,0,itoa(volt/1000,v,10),BigFont,NORM);
      printchar(84,0,'.',BigFont,NORM);
      writeString(94,0,itoa(((volt-(1000*(volt/1000))-volt%10)/10),v,10),BigFont,NORM);
      printchar(117,0,'V',BigFont,NORM);

      //https://blog.ampow.com/lipo-voltage-chart/
      //https://electronics.stackexchange.com/questions/435837/calculate-battery-percentage-on-lipo-battery
      if(volt>=3980){                   //over 3.98V, 100%
        volt=0; //Full (4 Segments)
      }else if(volt>=3840&&volt<3980){  //3.98-3.84V, 75%
        volt=1; //3 Segments
      }else if(volt>=3750&&volt<3840){  //3.84-3.75V, 50%
        volt=2; //2 Segments
      }else if(volt>=3690&&volt<3750){  //3.75-3.69V, 25%
        volt=3; //1 Segment
      }else{                            //under 3.69V, under 10%
        volt=4; //Low
      }
      
      if(digitalRead(CHG)){ //the on state charge indicator isn't currently used
        drawBMP(99,2,Battery,volt,NORM);    //Normal Indicator
      }else{
        drawBMP(99,2,Battery,Charge,NORM);  //Charging Indicator
      }
    }

    t=tme+totaltime; //32 bit unsigned long value representing total elapsed milliseconds during power on
    hours=t/3600000;
    minutes=(t/60000)%60;
    seconds=(t/1000)%60;
    if(hours<10){offset=45;}else if(hours<100){offset=39;}else{offset=33;}
    writeString(offset,0,itoa(hours,v,10),SmallFont,NORM);
    writeString(51,0,"h",SmallFont,NORM);
    writeString(45,1,"m",SmallFont,NORM);
    writeString(63,1,"s",SmallFont,NORM);
    if(minutes<10){offset=39;writeString(33,1," ",SmallFont,NORM);}else{offset=33;}
    writeString(offset,1,itoa(minutes,v,10),SmallFont,NORM);
    if(seconds<10){offset=57;writeString(51,1," ",SmallFont,NORM);}else{offset=51;}
    writeString(offset,1,itoa(seconds,v,10),SmallFont,NORM);

    cnt++;  //increment count used to count seconds
    delay(100);
  }
}
